from pathlib import Path
import toml
import importlib.util as importer


# Importing Local package
spec = importer.spec_from_file_location("context-manager", str(Path("../contextManager/contextManager.py").resolve()))
mariadb_context_manager = importer.module_from_spec(spec)
spec.loader.exec_module(mariadb_context_manager)


# Get Current Config
def get_configuration() -> dict[str, any]:
    # Load in configuration
    with open("config.toml", "r") as config:
        configuration = toml.load(config)
    mariadb_conn_params: dict[str, any] = {"user": configuration["database"]["username"], "password": configuration["database"]["password"], "host": configuration["server"]["host"], "port": configuration["server"]["port"], "database": configuration["database"]["database"]}
    return mariadb_conn_params


config = get_configuration()


# Test We can Connect
def test_connect():
    # connection = mariadb_context_manager.MariaDBCM(**config)
    test_query = "SHOW PROCESSLIST;"
    # results = connection.execute(test_query)
    #results = mariadb_context_manager.MariaDBCM(**config).execute(test_query)
    #for item in results:
    #    print(item)

    with mariadb_context_manager.MariaDBCM(**config) as con:
        result = con.execute(test_query)
        for r in result:
            print(r)


def main():
    test_connect()


if __name__ == "__main__":
    main()



import mariadb
from .conversions import conversions
from .combined_types import make_type_dictionary


# This will implement a context manager to work with MariaDB
class MariaDBCM:
    def __init__(
        self,
        host: str,
        user: str,
        password: str,
        database: str,
        port: int,
        buffered: bool = True,
        # Add functionality for converter
        converter: dict = None,
        return_dict: bool = False,
        prepared: bool = False,
        # Allows for loading infile
        allow_load_infile: bool = False,
    ):
        self.user = user
        self.password = password
        self.host = host
        self.port = port
        self.database = database
        self.buffered = buffered
        self.allow_load_infile = allow_load_infile
        # Makes our connection to mariadb
        self.conn = mariadb.connect(
            user=self.user,
            password=self.password,
            host=self.host,
            port=self.port,
            database=self.database,
            local_infile=self.allow_load_infile,
        )
        self.cur = self.conn.cursor()

    def __enter__(self):
        print(f"Connection to {self.database} was made")
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.conn.close()
        print("\nConnection has been closed...\n")
        if exc_type:
            print(f"exc_type: {exc_type}")
            print(f"exc_value: {exc_value}")
            print(f"traceback: {traceback}")
        return self

    def __check_connection_open(self) -> bool:
        if self.conn.cursor().closed:
            return False
        return True

    def __remove_comments(self, query: str) -> str:
        updated_query = ""
        for line in query.splitlines():
            if not (line.strip()).startswith("--"):
                updated_query += line.strip()

    def execute_change(self, statement: str, parameters: tuple) -> dict:
        if statement.strip() != "" and parameters is not None:
            ran_statement = self.cur.execute_many(statement, parameters)
            statement_results = {
                "statement": ran_statement.statement,
                "rows_updated": ran_statement.rowcount,
                "number_of_warnings": ran_statement.warnings,
            }
            return statement_results

    def execute(self, query: str) -> dict:
        result = {}
        if query.strip() != "":
            with self.conn as conn:
                cursor = conn.cursor(
                    dictionary=self.return_dict, prepared=self.prepared
                )
                cursor.execute(query)
                metadata = cursor.metadata
                if cursor.rowcount >= 0 and cursor.description:
                    result["data"] = cursor.fetchall()
                result["columns"] = metadata["field"]
                result["statement_ran"] = cursor.statement
                result["warnings"] = cursor.warnings
                result["rowcount"] = cursor.rowcount
                result["data_types"] = make_type_dictionary(
                    column_names=result["columns"], data_types=result["types"]
                )
        else:
            print("No query given")

        return result

    def execute_many(self, queries: str) -> list:
        results = []
        for query in queries.strip().split(";"):
            result = self.execute(query)
            results.append(result)
        return results

    def execute_stored_procedure(self, stored_procedure_name: str, inputs: tuple = ()):
        with self.conn as conn:
            cursor = conn.cursor(dictionary=self.return_dict, prepared=self.prepared)
            cursor.callproc(stored_procedure_name, inputs)
            result = {}
            metadata = cursor.metadata
            if cursor.sp_outparams:
                result["data"] = cursor.fetchall()
            result["columns"] = metadata["field"]
            result["warnings"] = cursor.warnings
            result["rowcount"] = cursor.rowcount
            result["data_types"] = make_type_dictionary(
                column_names=result["columns"], data_types=result["types"]
            )
        return result



from mariadb.constants import FIELD_TYPE
from typing import List, Dict


def convert(value) -> str:
    if (
        value == FIELD_TYPE.DECIMAL
        or value == FIELD_TYPE.FLOAT
        or value == FIELD_TYPE.DOUBLE
        or value == FIELD_TYPE.NEWDECIMAL
    ):
        return "float"
    elif (
        value == FIELD_TYPE.INT24
        or value == FIELD_TYPE.TINY
        or value == FIELD_TYPE.SHORT
        or value == FIELD_TYPE.LONG
        or value == FIELD_TYPE.LONGLONG
    ):
        return "int"
    elif (
        value == FIELD_TYPE.VAR_STRING
        or value == FIELD_TYPE.STRING
        or value == FIELD_TYPE.VARCHAR
    ):
        return "str"
    elif (
        value == FIELD_TYPE.DATE
        or value == FIELD_TYPE.TIME
        or value == FIELD_TYPE.DATETIME
        or value == FIELD_TYPE.YEAR
        or value == FIELD_TYPE.TIMESTAMP
        or value == FIELD_TYPE.NEWDATE
        or value == FIELD_TYPE.TIMESTAMP2
        or value == FIELD_TYPE.DATETIME2
        or value == FIELD_TYPE.TIME2
    ):
        return "str"
    else:
        return "str"


def make_type_dictionary(
    column_names: List[str], data_types: List[str]
) -> Dict[str, str]:
    values = list(map(convert, data_types))
    return {
        column_names: data_types
        for column_names, data_types in zip(column_names, values)
    }




from datetime import date, datetime, time
from mariadb.constants import FIELD_TYPE, INDICATOR

"""Create Conversion Functions"""
def convert_to_string(s):
    return str(s)

def convert_to_int(s):
    return int(s)

def convert_to_float(s):
    return float(s)

def convert_to_set(s):
    return {value for value in s}

def none_to_mariadb_none(s):
    return INDICATOR.NULL

conversions = {
     **{FIELD_TYPE.LONG: convert_to_float},
     **{FIELD_TYPE.NULL: none_to_mariadb_none},
     **{FIELD_TYPE.LONGLONG: convert_to_float},
}
